import time
from typing import List

import pygame
from pygame import Surface

from controllers.ControllerGame import ControllerGame
from controllers.ControllerTank import ControllerTank
from models.enums.EnumGameObjectDirection import EnumGameObjectDirection
from models.enums.EnumGameObjectType import EnumGameObjectType
from views.components.ComponentGameObject import ComponentGameObject, sprite_width
from views.components.ComponentTank import ComponentTank
from views.components.base import ComponentBase
from views.factories.FactoryGameObject import FactoryGameObject
from views.components.ComponentBullet import ComponentBullet

class WindowGame:
    def __init__(self):
        super().__init__()

        game = ControllerGame.new_game()
        self.screen = pygame.display.set_mode(
            (game.map_size[0] * sprite_width, game.map_size[1] * sprite_width)
        )
        self.is_game_running = True
        self.set_game(game)


    def set_game(self, game):
        self.game = game
        self.game_components: List[ComponentBase] = []
        self.player = [game_object for game_object in self.game.game_objects if
                       game_object.game_object_type == EnumGameObjectType.Enemy][0]

    def show(self):
        self.is_game_running = True
        # main game loop
        time_last = pygame.time.get_ticks()
        while self.is_game_running:
            # get delta seconds
            time_current = pygame.time.get_ticks()
            delta_milisec = time_current - time_last
            time_last = time_current

            # update
            self.update(delta_milisec)

            # draw
            self.render()

            # update display
            pygame.display.flip()
            time.sleep(0.01)

    def sync_game_components(self):
        # remove missing game_components
        existing_game_objects = []
        for game_component in self.game_components:
            if game_component.game_object not in self.game.game_objects:
                self.game_components.remove(game_component)
            else:
                existing_game_objects.append(game_component.game_object)

        # add new game_components
        for game_object in self.game.game_objects:
            if game_object not in existing_game_objects:
                if game_object.game_object_type == EnumGameObjectType.Enemy:
                    game_component = ComponentTank(game_object)
                    self.game_components.append(game_component)
                elif game_object.game_object_type == EnumGameObjectType.Bullet:
                    game_component = ComponentBullet(game_object)
                    self.game_components.append(game_component)
                else:
                    self.game_components.append(ComponentGameObject(game_object))


    def update(self, delta_milisec):
        ControllerGame.update(self.game, delta_milisec)

        self.sync_game_components()

        for game_component in self.game_components:
            game_component.update(delta_milisec)

        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                self.is_running = False
            if event.type == pygame.KEYDOWN:
                if event.key == pygame.K_ESCAPE:
                    self.is_game_running = False
                elif event.key == pygame.K_UP:
                    ControllerTank.set_direction(self.player, EnumGameObjectDirection.Up)
                elif event.key == pygame.K_DOWN:
                    ControllerTank.set_direction(self.player, EnumGameObjectDirection.Down)
                elif event.key == pygame.K_LEFT:
                    ControllerTank.set_direction(self.player, EnumGameObjectDirection.Left)
                elif event.key == pygame.K_RIGHT:
                    ControllerTank.set_direction(self.player, EnumGameObjectDirection.Right)
            elif event.type == pygame.KEYUP:
                if event.key in [pygame.K_UP, pygame.K_DOWN, pygame.K_LEFT, pygame.K_RIGHT]:
                    ControllerTank.set_direction(self.player, EnumGameObjectDirection.NotSet)
                if event.key == pygame.K_SPACE:
                    ControllerTank.fire(self.player, self.game)

    def render(self):
        self.screen.fill((0, 0, 0))
        for game_component in self.game_components:
            game_component.render(self.screen)

        font = pygame.font.SysFont('arial', 20)
        img_font = font.render(f"score: 0", True, (255, 255, 255))
        self.screen.blit(img_font, (0, 0))